#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2018-05-29
#
# Monitor check Point fragmented packets statistics
#
# 2020-06-07: rewrite check function, added wato
# 2020-06-08: changed snm-scan function
# 2021-08-24: rewritten for CMK 2.0
# 2023-ß5-29: moved gui file to ~/local/lib/check_mk/gui/plugins/..
#
# sample info
# [['2832', '0', '1416']]
#

import time
from dataclasses import dataclass
from typing import Optional

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    check_levels,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
    get_rate,
    GetRateError,
    get_value_store,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class CheckpointFragments:
    fwfrag_fragments: int
    fwfrag_expired: int
    fwfrag_packets: int


def parse_checkpoint_fragments(string_table: StringTable) -> Optional[CheckpointFragments]:
    try:
        fwfrag_fragments, fwfrag_expired, fwfrag_packets = string_table[0]
    except (IndexError, ValueError):
        return

    return CheckpointFragments(
        fwfrag_fragments=int(fwfrag_fragments),
        fwfrag_expired=int(fwfrag_expired),
        fwfrag_packets=int(fwfrag_packets),
    )


def discovery_checkpoint_fragments(section: CheckpointFragments) -> DiscoveryResult:
    yield Service()


def check_checkpoint_fragments(params, section: CheckpointFragments) -> CheckResult:
    now_time = time.time()
    value_store = get_value_store()

    for label, key, value, warncrit in [
        ('Fragmented packets/s', 'fwfrag_packets', section.fwfrag_packets, 'warncritpackets'),
        ('Fragments/s', 'fwfrag_fragments', section.fwfrag_fragments, 'warncritfragments'),
        ('Fragments expired/s', 'fwfrag_expired', section.fwfrag_expired, 'warncritexpired')
    ]:
        try:
            value = get_rate(value_store, f'checkpoint_fragments_{key}', now_time, value, raise_overflow=True)
        except GetRateError:
            value = 0
        yield from check_levels(
            value=value,
            label=label,
            metric_name=f'checkpoint_fragments_{key}',
            levels_upper=params.get(warncrit, (None, None)),
            render_func=lambda v: f'{v:.0f}'
        )


register.snmp_section(
    name='checkpoint_fragments',
    parse_function=parse_checkpoint_fragments,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.1.26.6',  # CHECKPOINT-MIB::fwFragments
        oids=[
            '1',  # fwFrag-fragments
            '2',  # fwFrag-expired
            '3',  # fwFrag-packets
        ]
    ),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_fragments',
    service_name='Fragments',
    discovery_function=discovery_checkpoint_fragments,
    check_function=check_checkpoint_fragments,
    check_default_parameters={
        # 'warncritpackets': (10, 20),
        # 'warncritfragments': (10, 20),
        # 'warncritexpired': (10, 20),
    },
    check_ruleset_name='checkpoint_fragments',
)
